import subprocess
import re

def get_version(command):
    try:
        # 使用 subprocess 调用系统命令获取版本信息
        output = subprocess.check_output(command, shell=True, stderr=subprocess.STDOUT, universal_newlines=True)
        # 使用正则表达式提取版本号
        version_match = re.search(r'(\d+\.\d+(\.\d+){0,2})', output)
        if version_match:
            return version_match.group(1)
        else:
            return False
    except subprocess.CalledProcessError:
        return False

def getGCCver():
    # 检查是否能执行 gcc 命令
    return get_version('gcc --version')

def getClangVer():
    # 检查是否能执行 clang 命令
    return get_version('clang --version')

def getMSVCver():
    # 检查是否能执行 cl 命令（MSVC编译器）
    return get_version('cl')
    
def getRustVer():
    process = subprocess.Popen(['rustc --version'], stdout=subprocess.PIPE, stderr=subprocess.PIPE,shell=True)
    stdout, stderr = process.communicate()
    if process.returncode == 0:
        rustVer = stdout.decode().split(' ')[1]
        return rustVer
    else:
        return False

print("GCC 版本:", getGCCver())
print("Clang 版本:", getClangVer())
print("MSVC 版本:", getMSVCver())
print('Rust版本:',getRustVer())


from . import util
from rich.console import Console
from rich.table import Table
    
def search(name):
    util.loadIndex() #加载索引    
    for Index in util.IndexList:
        dt = Index.packageList.get(name,None)
        if dt:
            return Index.IndexURL
            
def main(packlist,upgrade,reads,force_reinstall,ignore_requires_python,yes):
    console = Console()
    if reads: # 从文件读取列表
        f = open(packlist[0])
        packlist = f.read().split('\n')
        
    packsInfo = {}
    with console.status('🚀🔍 Searching for package information...') as status:
        table = Table(show_header=True,header_style='bold')
        table.add_column("Package Name", width=20,style='bold')
        table.add_column("Package Source", width=20,style='green')
      
        for rawpack in packlist: # 解析指定了版本的包名
            if '==' in rawpack:
                pack = rawpack.split('==')[0]
            elif '>=' in rawpack:
                pack = rawpack.split('>=')[0]
            elif '<=' in rawpack:
                pack = rawpack.split('<=')[0]
            elif '<' in rawpack:
                pack = rawpack.split('<')[0]
            elif '>' in rawpack:
                pack = rawpack.split('>')[0]
            else:
                pack = rawpack
                
            
            result = search(pack.lower()) # 转小写并获取源地址
            packsInfo[pack] = [result,rawpack]
         
    canInstallPack = []   
    for k,v in packsInfo.items():
        if not v[0]:
            table.add_row(k, "[red]Not found[red]")
        else:
            table.add_row(k,v[0])
            canInstallPack.append(k)
    console.print(table)
        
    if len(canInstallPack) < 1:
        console.print('❌ [red]There are no packages available for installation.[red]')
        exit()
        
    console.print('📦 Packages to be installed:')
    console.print(' '.join(canInstallPack))
           
    while True: # 是否允许安装
        if yes:
            break
               
        ye = console.input('Proceed with installation? [Y/n]: ')
            
        if ye.lower() == 'y':
            break
        elif ye.lower() == 'n':
            console.print('🛑 [red]User canceled the installation.[/red]')
            exit()
        else:
            continue
                
    console.print('🛠️ Initiating pip installation...')        
    for pack in canInstallPack:
            
        # 构建命
        args = [ '-i', packsInfo[pack][0]] # 指定源
                
        if upgrade: # 升级
            args.append('-U')
        if force_reinstall: # 强制重新安装
            args.append('--force-reinstall')
        if ignore_requires_python: # 忽略Python版本
            args.append('--ignore-requires-python')
                    
        args.append(packsInfo[pack][1])
        ret = util.runpip('install',args) # 运行pip
            
        if ret.returncode != 0: #是否执行完毕
            console.print('❌ [red]Installation failed.[/red]')
            exit()
        console.print(f'✅ [green]Installation successful for {pack}[/green]')
        
            
        


import requests
from moyanlib import jsons
from tqdm import tqdm
import dill
import os
import urllib.parse as urlparse
from hashlib import sha256
from .util import dirs
from rich.console import Console

console = Console()
def getSourceID(url):
    '''
    获取源id
    '''
    url = urlparse.urlparse(url)
    SourceID_Str = url.netloc + url.path # 取主机名和路径
    sha = sha256()
    sha.update(SourceID_Str.encode())
    return sha.hexdigest()[:8] # 取前八位
    
class Index:
    def __init__(self,indexurl):
        self.packageList = {}
        self.IndexURL = indexurl
        self.number = 0
        
    def addPackage(self, name):
        self.packageList[name] = 'H'
        self.number += 1
    
def getIndex(url):
    req = requests.get(url) # 请求HTML
    HTMLIndex = req.text

    ClassIndex = Index(url)

    console.print('🔍 Parsing HTML index...')
    for line in tqdm(HTMLIndex.split('\n')):
        # 提取并筛选标签
        line_list = line.split('>') 
        if len(line_list) > 1 and '<a ' in line:
            package_name = line_list[1].split('<')[0]
            
            ClassIndex.addPackage(package_name) # 添加包
    
    console.print('Total number of packages:', str(ClassIndex.number))
    console.print('📚 Saving index..."')     
    dill.dump(ClassIndex,open(f'{dirs.user_data_dir}/Index/{getSourceID(url)}.pidx','wb'))

def getAllIndex():
    '''
    SourceList = [
        'https://pypi.tuna.tsinghua.edu.cn/simple',
        'https://mirrors.bfsu.edu.cn/pypi/web/simple/'
    ]
    '''
    SourceList = jsons.load(open(os.path.join(dirs.user_config_dir,'Source.json'))) # 加载源列表
    if len(SourceList) < 1:
        console.print('❌ [red]You have not configured any sources.[/red]')
        exit(1)
        
    for url in SourceList: # 遍历源列表
        console.print('📚 Downloading index from', url+'...')
        getIndex(url)
        console.print('✅ [green]Index downloaded successfully![/green]')
        
